#include "uart_protocol.h"
#include "platform.h"
#include "mxchipWNet_HA.h"


#define ONE_SECOND 1000
#define ONE_MINUTE 60*ONE_SECOND

static u8 hugebuf[1500]; // cmd, fwd data are saved in this buffer

extern u32 MS_TIMER;
extern mxchipWNet_HA_st *device_info;


midea_cmd_head_t mideaNetCmd = {{0xAA, 0x55, 0xAA, 0x55}, 0x67, 0x98, 0x00, 0x00, 30, 0xAA, {0x00}};
midea_cmd_head_t mideaUartCmd = {{0xAA, 0x55, 0xAA, 0x55}, 0x98, 0x67, 0x00, 0x00, 30, 0x55, {0x00}};


static u8 calc_sum(void *data, u32 len)
{
  u8  cksum=0;
  u8 *p=data;

  while (len > 0)
  {
    cksum += *p++;
    len --;
  }

  cksum = ~cksum;
  cksum++;

  return cksum;
}

static void mac_str2int(char *str, u8 *mac)
{
  int i, j;
  char mac_tmp[12];

  for(i=0, j=0; i<12; i++,j++){
    mac_tmp[i] = str[j];
    if(str[j+1]==':')
      j++;
  }
  str2hex((u8 *)mac_tmp, (u8 *)mac, 6);
}

void status_changed_callback(u8 status) //used in uart protocol v2
{
  u8 i, tmp;
  midea_cmd_head_t *cmd = (midea_cmd_head_t *)hugebuf;
  memset(cmd, 0x0, sizeof(midea_cmd_head_t));
  memcpy(cmd, &mideaUartCmd, sizeof(midea_cmd_head_t));
  *(cmd->data+BYTE_TYPE) = 0x1F;
	*(cmd->data+BYTE_RETURN1) = 0x0;
  *(cmd->data+BYTE_RETURN2) = status;
  if (is_network_state(REMOTE_CONNECT))
    *(cmd->data+BYTE_RETURN3) = 3;
  else if (is_network_state(STA_CONNECT))
    *(cmd->data+BYTE_RETURN3) = 1;
  else 
    *(cmd->data+BYTE_RETURN3) = 2;
  for(i = BYTE_TYPE; i < BYTE_CHKSUM; i++)
    tmp += *(cmd->data + i);
  tmp = ~tmp + 1;
  *(cmd->data+BYTE_CHKSUM) = tmp;

  hal_uart_send_data((u8 *)cmd, PACKAGE_HEAD_LEN + PACKAGE_CHKSUM_LEN + cmd->datalen);
	hal_uart_tx_tick();
}

void uart_report_state() //used in uart protocol v1
{
  u8 *buf = hugebuf;
  u8 state;
  u8 midea_mac[6];

  if (is_network_state(REMOTE_CONNECT))
    state = 3;
  else if (is_network_state(STA_CONNECT))
    state = 1;
  else 
    state = 2;
  
  buf[0] = 0xAA;
  buf[1] = 0xA1;

  mac_str2int((char*)(device_info->status.mac), midea_mac);

	memcpy(&buf[2], midea_mac, 6);
  memset(&buf[8], 0, 10);
  buf[18] = 0xc3;
  buf[19] = state;
  buf[20] = calc_sum(&buf[1], 19);
  buf[21] = 0x55;
  
    
  hal_uart_send_data(buf, 22);
	hal_uart_tx_tick();
}

int uart_cmd_process(u8 *buf, int len)
{
  u8 i, tmp = 0;
  midea_cmd_head_t *cmd_header;
    
  cmd_header = (midea_cmd_head_t *)buf;

  if(cmd_header->cmd_identifier1 != 0x98 || cmd_header->cmd_identifier2 != 0x67)
    return 0;

  if(cmd_header->lead_code != 0x55)
    return 0;

  if(*(cmd_header->data+BYTE_TYPE) == 0xF1){   //UART command  
    if(*(cmd_header->data+BYTE_CMD)&(1<<BIT_INIT)){
      restoreConfiguration();
      delay_reload();
      *(cmd_header->data+BYTE_RETURN1) = 0;
      *(cmd_header->data+BYTE_RETURN1) |= 1<<BIT_INIT;
    }
    else if(*(cmd_header->data+BYTE_CMD)&(1<<BIT_RESET)){
      delay_reload();
      *(cmd_header->data+BYTE_RETURN1) = 0;
      *(cmd_header->data+BYTE_RETURN1) |= 1<<BIT_RESET;
    }
    else if(*(cmd_header->data+BYTE_CMD)&(1<<BIT_NET)){
      *(cmd_header->data+BYTE_RETURN1) = 0;
      *(cmd_header->data+BYTE_RETURN1) |= 1<<BIT_NET;
    }
    else if(*(cmd_header->data+BIT_STATUS)&(1<<BIT_NET)){
      *(cmd_header->data+BYTE_RETURN1) = 0;
      *(cmd_header->data+BYTE_RETURN1) |= 1<<BIT_NET;
    }

		else if(device_info->mode != Type_AUTO && MS_TIMER <3*ONE_MINUTE){
      if(*(cmd_header->data+BYTE_CMD)&(1<<BIT_STA)){
        /*Record the wifi mode in MANUAL mode*/
        if(device_info->mode == Type_MANUAL){
          device_info->conf.interface = Station;
          updateConfiguration(device_info);
        }
        sta_start();
      }
      else {
        sta_close();
      }

      if(*(cmd_header->data+BYTE_CMD)&(1<<BIT_AP)){
        /*Record the wifi mode in MANUAL mode*/
        if(device_info->mode == Type_MANUAL){
          device_info->conf.interface = Soft_AP;
          updateConfiguration(device_info);
        }
        uap_start();
      }
      else{
        uap_close();
      }

      if(*(cmd_header->data+BYTE_CMD)&(1<<BIT_CONFIG)){

      }
      else{

      }
    }

    *(cmd_header->data+BYTE_TYPE) = 0x1F;
    get_status_uart(cmd_header->data+BYTE_RETURN2);
    if (is_network_state(REMOTE_CONNECT))
      *(cmd_header->data+BYTE_RETURN3) = 3;
    else if (is_network_state(STA_CONNECT))
      *(cmd_header->data+BYTE_RETURN3) = 1;
    else 
      *(cmd_header->data+BYTE_RETURN3) = 2;

    for(i = BYTE_TYPE; i < BYTE_CHKSUM; i++)
      tmp += *(cmd_header->data + i);
    tmp = ~tmp + 1;
    *(cmd_header->data+BYTE_CHKSUM) = tmp;

    hal_uart_send_data(buf, PACKAGE_HEAD_LEN + PACKAGE_CHKSUM_LEN + cmd_header->datalen);
  }
  else if(*(cmd_header->data+BYTE_TYPE) == 0xE1){  //Midea type return 
    return *(cmd_header->data+BYTE_TYPE_H)<<2 + *cmd_header->data+BYTE_TYPE_L;
  }    
  else{   //UART transparent data
    wifi_output(buf, len);
  }
  return 0;
}


int uart_cmd_process_legacy(u8 *buf, int len)
{
  static int wifi_disabled = 0;
  int control;
  u8 midea_mac[6] = {0};
#define CTL_GOTO_DEFAULT 1<<0
#define CTL_DISABLE_WIFI 1<<1

  if (len == 31) { // transport
		mac_str2int((char*)(device_info->status.mac), midea_mac);
    memcpy(&buf[1], midea_mac, 6);
    memset(&buf[7], 0, 10);
    if(is_network_state(UAP_START)==1)
      buf[21] &= 0x7f;
    else
      buf[21] |= 0x80;
		buf[29] = calc_sum(&buf[1], 28);
    wifi_output(buf, len);
  } else if (len == 22) { // handshak
    if (buf[18] != 0xc3)
      return 0;
    control = buf[19];
    
    if (control & CTL_GOTO_DEFAULT) {
      restoreConfiguration();
      delay_reload();
    } 
    
    if (control & CTL_DISABLE_WIFI) { // user request to disable wifi
      if (wifi_disabled == 0) {
        wifi_disabled = 1;
        wifi_disable();
      }
    } else {
      if (wifi_disabled == 1) { // user request to open wifi
        delay_reload();
      }
    }
    uart_report_state();
  }

  return 0;
}


void uart_tick(void)
{
  int recvlen; 

  recvlen = hal_uart_get_one_packet(hugebuf, device_info->protocolVer);
  if (recvlen == 0)
    return; 

  if(device_info->protocolVer == 1){
    uart_cmd_process_legacy(hugebuf, recvlen);
  }
  else{
    uart_cmd_process(hugebuf, recvlen);
  }
}

midea_type get_midea_type(void)
{
  u8 i, tmp;
  int recvlen; 
  u32 uart_send_tick = 0;
  midea_type type = Type_unknow;

  while(MS_TIMER<6000){  //excute in the 6 seconds after bootup
    if(MS_TIMER>uart_send_tick){
      midea_cmd_head_t *cmd = (midea_cmd_head_t *)hugebuf;
  	  memset(cmd, 0x0, sizeof(midea_cmd_head_t));
      memcpy(cmd, &mideaUartCmd, sizeof(midea_cmd_head_t));
      *(cmd->data+BYTE_TYPE) = 0x91;
      for(i = BYTE_TYPE; i < BYTE_CHKSUM; i++)
        tmp += *(cmd->data + i);
      tmp = ~tmp + 1;
      *(cmd->data+BYTE_CHKSUM) = tmp;

      hal_uart_send_data((u8 *)cmd, PACKAGE_HEAD_LEN + PACKAGE_CHKSUM_LEN + cmd->datalen);
      uart_send_tick = MS_TIMER + 1000;
    }
		hal_uart_tx_tick();

    recvlen = hal_uart_get_one_packet(hugebuf, 2);
    if (recvlen == 0)
      continue; 
    type = (midea_type)uart_cmd_process(hugebuf, recvlen);
    if( type == 0)
      continue;
    return type;
  } 
	return type;
}
