#ifndef __MXCHIPWNET_HA_H
#define __MXCHIPWNET_HA_H

#include "stm32f2xx.h"
#include "mxchipWNet.h"

#define FW_VERSION 0x31621101

#define CONFIGURATION_VERSION 0x0000003// if changed default configuration, add this num
#define FIRMWARE_VERSION 0x0000001

#define DEFAULT_UAP_SSID "mideaktjy"
#define DEFAULT_UAP_KEY "jykt2012"
#define DEFAULT_STA_SSID "jykt"
#define DEFAULT_STA_KEY "ktjy2012"
#define DEFAULT_REMOTE_ADDR "iot.midea.com.cn"
//#define DEFAULT_REMOTE_ADDR "192.168.2.125"
#define DEFAULT_REMOTE_PORT 10000
#define TCP_SERVER_PORT 10000 //WIFI create a TCP server listen on this port
#define UDP_BROADCAST_PORT 8090 // UDP broadcast port
#define DEFAULT_MIDEA_BOARD_TYPE  Type_RD_AUTO
#define DEFAULT_WIFI_INTERFACE Station

#define UAP_IP_ADDRESS "10.10.10.1" // This is the uAP IP address
#define UAP_MASK_ADDRESS "10.10.10.1" 
#define UAP_GW_ADDRESS "10.10.10.1" 
#define UAP_DNS_ADDRESS "10.10.10.1" 
#define UAP_DHCP_SERVER_START "10.10.10.100" 
#define UAP_DHCP_SERVER_END "10.10.10.200" 

#define CLOUD_ENABLE 1
#define CLOUD_RETRY 30*ONE_SECOND
#define CLOUD_HEART_INTERVAL 600*ONE_SECOND
#define UAP_START_TIME 26*ONE_SECOND // uAP start time. If wifi module can't connect to AP for 20 seconds, uAP is startup.
#define CLOUD_SSL

enum {
  CMD_READ_VERSION = 1, 
  CMD_READ_CONFIG, 
  CMD_WRITE_CONFIG, 
  CMD_SCAN, 
  CMD_OTA, 
  CMD_NET2COM,
  CMD_COM2NET, 
  CMD_GET_STATUS, 
  CMD_CONTROL,    
  CMD_SEARCH, 
};

enum {
  STA_CONNECT = 1<<0,
  //UAP_SERVER_START = 1<<1,
  UAP_START = 1<<2,
  REMOTE_CONNECT = 1<<3,
};


typedef enum {
  Type_unknow = 0x0,
  Type_Native = 0x1001, 
  Type_Oversea = 0x2001, 
  Type_Oversea_FORCE = 0x2002, 
  Type_Oversea_MANUAL = 0x2003, 
  Type_RD_AUTO = 3001
} midea_type;

typedef enum {
  Type_AUTO,
  Type_FORCE, 
  Type_MANUAL
} operation_mode;


#define BYTE_TYPE       10
#define BYTE_CMD        11
#define BYTE_CHKSUM     29

#define BIT_INIT        0
#define BIT_RESET       1
#define BIT_NET         2
#define BIT_STA         3
#define BIT_AP          4
#define BIT_CONFIG      5
#define BIT_STATUS      6
#define BYTE_RETURN1    11
#define BYTE_RETURN2    12
#define BYTE_RETURN3    13
#define BYTE_TYPE_H      12
#define BYTE_TYPE_L      13

#define PACKAGE_HEAD_LEN 10
#define PACKAGE_CHKSUM_LEN 1

#pragma pack(1)
typedef struct _midea_cmd_head {
  u8 identifier[4];// Allways 0xAA, 0x55, 0xAA, 0x55
  u8 cmd_identifier1; 
  u8 cmd_identifier2; 
  u8 protocol_ver;
  u8 protocol_sub_ver;
  u8 datalen;
  u8 lead_code;
  u8 data[1]; 
}midea_cmd_head_t;

typedef struct _cloud_login {
  u8 lead_code;
  u8 version;
  u8 mac[12]; 
  u8 security[16];
  u8 ssid[32];
  u8 bssid[12];
} cloud_login_t;

#pragma pack(1)
typedef struct _mxchip_cmd_head {
  u16 flag; // Allways BB 00
  u16 cmd; // commands, return cmd=cmd|0x8000
  u16 cmd_status; //return result
  u16 datalen; 
  u8 data[1]; 
}mxchip_cmd_head_t;

#pragma pack(1)
typedef struct _current_state_ {
  u32 uap_state;
  u32 sta_state;
  u32 tcp_client;
  u32 signal;
  u8 ip[16];
  u8 mask[16];
  u8 gw[16];
  u8 dns[16];
  u8 mac[18];
}current_state_t;

#pragma pack(1)
typedef struct _mxchip_state_ {
  u16 flag; 
  u16 cmd; 
  u16 cmd_status; 
  u16 datalen; 
  current_state_t status;
  u16 cksum;
}mxchip_state_t;

#pragma pack(1)
typedef struct _mxchipWNet_config_st {
  /*OTA options*/
  boot_table_t bootTable;
  /*EABox configuration*/
  u32 magicNumber;
  u32 versionNumber;
  midea_type mideaType;
  WiFi_Interface interface;   //Save the last wifi mode is manual mode 

  char uap_ssid[32];
  char uap_key[32];
  char sta_ssid[32];  
  char sta_key[32]; 

  u8 server_domain[64];
  u32 server_port;
  u32 dhcp_enable;
  u8 ip[16];
  u8 mask[16];
  u8 gw[16];
  u8 dns[16];
}mxchipWNet_HA_config_st;

#define CONFIG_DATA_SIZE (sizeof(mxchipWNet_HA_config_st)-16-sizeof(boot_table_t))

typedef struct _mxchipWNet_st
{
  mxchipWNet_HA_config_st conf; 
  current_state_t status;
  int protocolVer;
  operation_mode mode;
  int *tcpClient_fd;
  int tcpServer_fd;
  int cloud_fd;
  int udpSearch_fd;
}  mxchipWNet_HA_st;


/* ======MXCHIP lib API ==========*/
/* Sysytem initilize */
void mxchipWNet_HA_init(void) ;
/* main tick */
void mxchipWNet_HA_tick(void);
/* Get system status */
void get_status(mxchip_state_t *cmd);
void get_status_uart(u8 *status);
/* Reload*/
void system_reload(void);
/* Back to default configuration and reload */
void system_default(void);
/* Start WPS*/
int wps_pbc_start(void);
/* Wifi Power down*/
void wifi_disable(void);
/* Output data by wifi */
void wifi_output(u8*buf, int len);
/* Start AP */
void uap_start(void);
/* Close AP */
void uap_close(void);
/* Start Station */
void sta_start(void);
/* Close Station */
void sta_close(void);

void delay_reload(void);

int is_network_state(int state);


/*=====User define callback function ====*/
void status_changed_callback(u8 status);
void uart_report_state(void);
void wps_fail(void);
void wps_success(u8*ssid, u8*key);
void wifi_input(u8*buf, int len);

#endif
