#include "can.h"


CAN_InitTypeDef        CAN_InitStructure;
CAN_FilterInitTypeDef  CAN_FilterInitStructure;
CanTxMsg TxMessage;

/* Private variables ---------------------------------------------------------*/
uint16_t CAN1_ID;
uint8_t CAN1_DATA0,CAN1_DATA1,CAN1_DATA2,CAN1_DATA3,CAN1_DATA4,CAN1_DATA5,CAN1_DATA6,CAN1_DATA7;
__IO uint8_t Can1Flag,Can2Flag;


/**
  * @brief  Configures the CAN.
  * @param  None
  * @retval None
  */
void CAN1_Config(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  
  /* CAN GPIOs configuration **************************************************/

  /* Enable  clock */
  RCC_AHB1PeriphClockCmd(USE_CANx_TXD_CLK | USE_CANx_RXD_CLK, ENABLE);
  
	
  /* Connect CAN pins */
  GPIO_PinAFConfig(USE_CANx_TXD_PORT, USE_CANx_TXD_SOURCE, USE_CANx_AF);
  GPIO_PinAFConfig(USE_CANx_RXD_PORT, USE_CANx_RXD_SOURCE, USE_CANx_AF); 
  
  /* Configure CAN RX and TX pins */
  GPIO_InitStructure.GPIO_Pin = USE_CANx_TXD_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
  GPIO_Init(USE_CANx_TXD_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = USE_CANx_RXD_PIN;
  GPIO_Init(USE_CANx_RXD_PORT, &GPIO_InitStructure);


  USE_CANx_INIT(USE_CANx_CLK, ENABLE);
  /* CAN register init */
  CAN_DeInit(USE_CANx);
//  CAN_StructInit(&CAN_InitStructure);

  /* CAN cell init */
  CAN_InitStructure.CAN_TTCM = DISABLE;
  CAN_InitStructure.CAN_ABOM = DISABLE;
  CAN_InitStructure.CAN_AWUM = DISABLE;
  CAN_InitStructure.CAN_NART = DISABLE;
  CAN_InitStructure.CAN_RFLM = DISABLE;
  CAN_InitStructure.CAN_TXFP = DISABLE;
  CAN_InitStructure.CAN_Mode = CAN_Mode_Normal;
  CAN_InitStructure.CAN_SJW = CAN_SJW_1tq;
    
  /* CAN Baudrate = 1MBps (CAN clocked at 30 MHz) */
  CAN_InitStructure.CAN_BS1 = CAN_BS1_6tq;
  CAN_InitStructure.CAN_BS2 = CAN_BS2_8tq;
  CAN_InitStructure.CAN_Prescaler = 2;
  CAN_Init(USE_CANx, &CAN_InitStructure);

  /* CAN filter init */
  CAN_FilterInitStructure.CAN_FilterNumber = 0;
  CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;
  CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;
  CAN_FilterInitStructure.CAN_FilterIdHigh = 0x0000;
  CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;
  CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0000;
  CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;
  CAN_FilterInitStructure.CAN_FilterFIFOAssignment = 0;
  CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
  CAN_FilterInit(&CAN_FilterInitStructure);
  
  /* Transmit Structure preparation */
  TxMessage.StdId = 0x321;
  TxMessage.ExtId = 0x01;
  TxMessage.RTR = CAN_RTR_DATA;
  TxMessage.IDE = CAN_ID_STD;
  TxMessage.DLC = 1;
  
  /* Enable FIFO 0 message pending Interrupt */
  CAN_ITConfig(USE_CANx, CAN_IT_FMP0, ENABLE);
}


void NVIC_Config(void)
{
  NVIC_InitTypeDef  NVIC_InitStructure;

  NVIC_InitStructure.NVIC_IRQChannel = CAN1_RX0_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}

/*******************************************************************************
* Function Name  : Can1WriteData
* Description    : Can1 Write Date to CAN-BUS
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void Can1WriteData(uint16_t ID)
{
  CanTxMsg TxMessage;

  CAN1_DATA0=rand()%0xff;  CAN1_DATA1=rand()%0xff;  
  CAN1_DATA2=rand()%0xff;  CAN1_DATA3=rand()%0xff;  
  CAN1_DATA4=rand()%0xff;  CAN1_DATA5=rand()%0xff;
  CAN1_DATA6=rand()%0xff;  CAN1_DATA7=rand()%0xff; 

  /* transmit */
  TxMessage.StdId = ID;

  TxMessage.RTR = CAN_RTR_DATA;
  TxMessage.IDE = CAN_ID_STD;
  TxMessage.DLC = 8;
  TxMessage.Data[0] = CAN1_DATA0;    
  TxMessage.Data[1] = CAN1_DATA1;    
  TxMessage.Data[2] = CAN1_DATA2;    
  TxMessage.Data[3] = CAN1_DATA3;    
  TxMessage.Data[4] = CAN1_DATA4;    
  TxMessage.Data[5] = CAN1_DATA5;     
  TxMessage.Data[6] = CAN1_DATA6;    
  TxMessage.Data[7] = CAN1_DATA7;   
	printf("\r\n****************************************************************\r\n");
	printf("CAN Send Data \r\n");
	printf("CAN ID %x \r\n",CAN1_ID);
	printf("CAN_DATA0 %x \r\n",CAN1_DATA0);
	printf("CAN_DATA1 %x \r\n",CAN1_DATA1);
	printf("CAN_DATA2 %x \r\n",CAN1_DATA2);
	printf("CAN_DATA3 %x \r\n",CAN1_DATA3);
	printf("CAN_DATA4 %x \r\n",CAN1_DATA4);
	printf("CAN_DATA5 %x \r\n",CAN1_DATA5);
	printf("CAN_DATA6 %x \r\n",CAN1_DATA6);
	printf("CAN_DATA7 %x \r\n",CAN1_DATA7);
			
  CAN_Transmit(CAN1,&TxMessage);
}
