#include "spi_flash.h"


void SPI_Configuration(void)
{
	SPI_InitTypeDef SPI_InitStruct;	 
	GPIO_InitTypeDef GPIO_InitStructure;
	
	RCC_AHB1PeriphClockCmd(USE_SPIx_MOSI_CLK | USE_SPIx_MISO_CLK | USE_SPIx_SCK_CLK,ENABLE);
	USE_SPIx_INIT(USE_SPIx_CLK,ENABLE);
	
	GPIO_PinAFConfig(USE_SPIx_MOSI_PORT, USE_SPIx_MOSI_SOURCE,  USE_SPIx_AF);
	GPIO_PinAFConfig(USE_SPIx_MISO_PORT, USE_SPIx_MISO_SOURCE, USE_SPIx_AF);
	GPIO_PinAFConfig(USE_SPIx_SCK_PORT, USE_SPIx_SCK_SOURCE, USE_SPIx_AF);
	
	GPIO_InitStructure.GPIO_Pin = USE_SPIx_MOSI_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_DOWN;  
	GPIO_Init(USE_SPIx_MOSI_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = USE_SPIx_MISO_PIN;
	GPIO_Init(USE_SPIx_MISO_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = USE_SPIx_SCK_PIN;
	GPIO_Init(USE_SPIx_SCK_PORT, &GPIO_InitStructure);

	SPI_I2S_DeInit(USE_SPIx);
	SPI_InitStruct.SPI_Direction= SPI_Direction_2Lines_FullDuplex;
	SPI_InitStruct.SPI_DataSize = SPI_DataSize_8b; 
	SPI_InitStruct.SPI_Mode = SPI_Mode_Master;
	SPI_InitStruct.SPI_CPOL = SPI_CPOL_Low ;
	SPI_InitStruct.SPI_CPHA = SPI_CPHA_1Edge;
	SPI_InitStruct.SPI_NSS = SPI_NSS_Soft ;
	SPI_InitStruct.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_128;
	SPI_InitStruct.SPI_FirstBit = SPI_FirstBit_MSB;
	SPI_InitStruct.SPI_CRCPolynomial = 7;
	SPI_Init(USE_SPIx, &SPI_InitStruct);

	SPI_Cmd(USE_SPIx, ENABLE);

}

void SPIx_Send_byte(u16 data)
{
	while(SPI_I2S_GetFlagStatus(USE_SPIx, SPI_I2S_FLAG_TXE)==RESET);
	SPI_I2S_SendData(USE_SPIx,data);

	while(SPI_I2S_GetFlagStatus(USE_SPIx, SPI_I2S_FLAG_RXNE)==RESET);
	SPI_I2S_ReceiveData(USE_SPIx);
}

u16 SPIx_Receive_byte(void)
{
	while(SPI_I2S_GetFlagStatus(USE_SPIx, SPI_I2S_FLAG_TXE)==RESET);
	SPI_I2S_SendData(USE_SPIx,0x00);
	
	while(SPI_I2S_GetFlagStatus(USE_SPIx, SPI_I2S_FLAG_RXNE)==RESET);
	return SPI_I2S_ReceiveData(USE_SPIx);
}


void AT45DBXX_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	SPI_Configuration();

	printf("SPI is ready!\r\n");

  RCC_AHB1PeriphClockCmd(USE_SPIx_SSN_CLK,ENABLE);

  GPIO_InitStructure.GPIO_Pin = USE_SPIx_SSN_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;

  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(USE_SPIx_SSN_PORT, &GPIO_InitStructure);
  	
  AT45DBXX_Disable;
	
}


static void AT45DBXX_BUSY(void)
{
	u8 AT45DBXXStruct;
	AT45DBXX_Enable;
	SPIx_Send_byte(READ_STATE_REGISTER);
	do
	{AT45DBXXStruct = SPIx_Receive_byte();}
	while(!(AT45DBXXStruct & 0x80));
	AT45DBXX_Disable;	
}

void AT45DBXX_Read_ID(u8 *IData)
{
	u8 i;
	AT45DBXX_BUSY();
	AT45DBXX_Enable;
	SPIx_Send_byte(Read_ID); //ִжȡid		  	
  	for(i=0;i<4;i++)
  	{
  		IData[i] = SPIx_Receive_byte();
  	}
  	AT45DBXX_Disable;
}

void write_buffer(u16 BufferOffset,u8 Data)
{			
	AT45DBXX_Enable;
	SPIx_Send_byte(0x84);			
	SPIx_Send_byte(0xff);
	SPIx_Send_byte((u8)BufferOffset>>8);
	SPIx_Send_byte((u8)BufferOffset);		
	SPIx_Send_byte(Data);
	AT45DBXX_Disable;
}

u8 read_buffer(u16 BufferOffset)
{		
	u8 temp=0;

	AT45DBXX_Enable;
 	SPIx_Send_byte(0xd4);
	SPIx_Send_byte(0xff);
	SPIx_Send_byte((u8)BufferOffset>>8);
	SPIx_Send_byte((u8)BufferOffset);  
	SPIx_Send_byte(0xff);	
	temp=SPIx_Receive_byte();
	AT45DBXX_Disable;
	return temp;		
}

